@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2017 David R. MacIver
+# Most of this work is copyright (C) 2013-2018 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -29,8 +29,11 @@ import hypothesis.extra.numpy as npst
 import hypothesis.internal.conjecture.utils as cu
 from hypothesis.errors import InvalidArgument
 from hypothesis.control import reject
+from hypothesis.strategies import check_strategy
 from hypothesis.internal.compat import hrange
-from hypothesis.internal.branchcheck import check, check_function
+from hypothesis.internal.coverage import check, check_function
+from hypothesis.internal.validation import check_type, try_convert, \
+    check_valid_size, check_valid_interval
 
 try:
     from pandas.api.types import is_categorical_dtype
@@ -40,6 +43,10 @@ except ImportError:  # pragma: no cover
             return False
         return dt == 'category'
 
+if False:
+    from typing import Any, Union, Sequence, Set  # noqa
+    from hypothesis.searchstrategy.strategies import Ex  # noqa
+
 
 def dtype_for_elements_strategy(s):
     return st.shared(
@@ -63,7 +70,7 @@ def elements_and_dtype(elements, dtype, source=None):
         prefix = '%s.' % (source,)
 
     if elements is not None:
-        st.check_strategy(elements, '%selements' % (prefix,))
+        check_strategy(elements, '%selements' % (prefix,))
     else:
         with check('dtype is not None'):
             if dtype is None:
@@ -78,7 +85,7 @@ def elements_and_dtype(elements, dtype, source=None):
                     prefix,
                 ))
 
-    dtype = st.try_convert(np.dtype, dtype, 'dtype')
+    dtype = try_convert(np.dtype, dtype, 'dtype')
 
     if elements is None:
         elements = npst.from_dtype(dtype)
@@ -146,6 +153,7 @@ DEFAULT_MAX_SIZE = 10
 @st.cacheable
 @st.defines_strategy
 def range_indexes(min_size=0, max_size=None):
+    # type: (int, int) -> st.SearchStrategy[pandas.RangeIndex]
     """Provides a strategy which generates an :class:`~pandas.Index` whose
     values are 0, 1, ..., n for some n.
 
@@ -154,18 +162,23 @@ def range_indexes(min_size=0, max_size=None):
     * min_size is the smallest number of elements the index can have.
     * max_size is the largest number of elements the index can have. If None
       it will default to some suitable value based on min_size.
-
     """
-    st.check_valid_interval(min_size, max_size, 'min_size', 'max_size')
+    check_valid_size(min_size, 'min_size')
+    check_valid_size(max_size, 'max_size')
     if max_size is None:
-        max_size = min_size + DEFAULT_MAX_SIZE
+        max_size = min([min_size + DEFAULT_MAX_SIZE, 2 ** 63 - 1])
+    check_valid_interval(min_size, max_size, 'min_size', 'max_size')
     return st.integers(min_size, max_size).map(pandas.RangeIndex)
 
 
 @st.cacheable
 @st.defines_strategy
 def indexes(
-    elements=None, dtype=None, min_size=0, max_size=None, unique=True,
+    elements=None,  # type: st.SearchStrategy[Ex]
+    dtype=None,  # type: Any
+    min_size=0,  # type: int
+    max_size=None,  # type: int
+    unique=True,  # type: bool
 ):
     """Provides a strategy for producing a :class:`pandas.Index`.
 
@@ -185,10 +198,11 @@ def indexes(
       should pass a max_size explicitly.
     * unique specifies whether all of the elements in the resulting index
       should be distinct.
-
     """
-    st.check_valid_interval(min_size, max_size, 'min_size', 'max_size')
-    st.check_type(bool, unique, 'unique')
+    check_valid_size(min_size, 'min_size')
+    check_valid_size(max_size, 'max_size')
+    check_valid_interval(min_size, max_size, 'min_size', 'max_size')
+    check_type(bool, unique, 'unique')
 
     elements, dtype = elements_and_dtype(elements, dtype)
 
@@ -199,7 +213,14 @@ def indexes(
 
 
 @st.defines_strategy
-def series(elements=None, dtype=None, index=None, fill=None, unique=False):
+def series(
+    elements=None,  # type: st.SearchStrategy[Ex]
+    dtype=None,  # type: Any
+    index=None,  # type: st.SearchStrategy[Union[Sequence, pandas.Index]]
+    fill=None,  # type: st.SearchStrategy[Ex]
+    unique=False,  # type: bool
+):
+    # type: (...) -> st.SearchStrategy[pandas.Series]
     """Provides a strategy for producing a :class:`pandas.Series`.
 
     Arguments:
@@ -232,12 +253,11 @@ def series(elements=None, dtype=None, index=None, fill=None, unique=False):
         >>> series(dtype=int).example()
         0   -2001747478
         1    1153062837
-
     """
     if index is None:
         index = range_indexes()
     else:
-        st.check_strategy(index)
+        check_strategy(index)
 
     elements, dtype = elements_and_dtype(elements, dtype)
     index_strategy = index
@@ -286,7 +306,6 @@ class column(object):
     * fill: A default value for elements of the column. See
       :func:`~hypothesis.extra.numpy.arrays` for a full explanation.
     * unique: If all values in this column should be distinct.
-
     """
 
     name = attr.ib(default=None)
@@ -297,7 +316,11 @@ class column(object):
 
 
 def columns(
-    names_or_number, dtype=None, elements=None, fill=None, unique=False
+    names_or_number,  # type: Union[int, Sequence[str]]
+    dtype=None,  # type: Any
+    elements=None,  # type: st.SearchStrategy[Ex]
+    fill=None,  # type: st.SearchStrategy[Ex]
+    unique=False,  # type: bool
 ):
     """A convenience function for producing a list of :class:`column` objects
     of the same general shape.
@@ -307,12 +330,11 @@ def columns(
     objects, or a number, in which case that many unnamed columns will
     be created. All other arguments are passed through verbatim to
     create the columns.
-
     """
-    try:
+    if isinstance(names_or_number, (int, float)):
+        names = [None] * names_or_number  # type: list
+    else:
         names = list(names_or_number)
-    except TypeError:
-        names = [None] * names_or_number
     return [
         column(
             name=n, dtype=dtype, elements=elements, fill=fill, unique=unique
@@ -322,8 +344,11 @@ def columns(
 
 @st.defines_strategy
 def data_frames(
-    columns=None, rows=None, index=None
+    columns=None,  # type: Sequence[column]
+    rows=None,  # type: st.SearchStrategy[Union[dict, Sequence[Any]]]
+    index=None,  # type: st.SearchStrategy[Ex]
 ):
+    # type: (...) -> st.SearchStrategy[pandas.DataFrame]
     """Provides a strategy for producing a :class:`pandas.DataFrame`.
 
     Arguments:
@@ -425,13 +450,11 @@ def data_frames(
       dicts are passed, if there are keys with no corresponding column name,
       if sequences are passed if there are too many items) will result in
       InvalidArgument being raised.
-
     """
-
     if index is None:
         index = range_indexes()
     else:
-        st.check_strategy(index)
+        check_strategy(index)
 
     index_strategy = index
 
@@ -448,7 +471,7 @@ def data_frames(
                 @check_function
                 def row():
                     result = draw(rows)
-                    st.check_type(Iterable, result, 'draw(row)')
+                    check_type(Iterable, result, 'draw(row)')
                     return result
 
                 if len(index) > 0:
@@ -465,13 +488,13 @@ def data_frames(
             return rows_only()
 
     assert columns is not None
-    columns = st.try_convert(tuple, columns, 'columns')
+    cols = try_convert(tuple, columns, 'columns')  # type: Sequence[column]
 
     rewritten_columns = []
-    column_names = set()
+    column_names = set()  # type: Set[str]
 
-    for i, c in enumerate(columns):
-        st.check_type(column, c, 'columns[%d]' % (i,))
+    for i, c in enumerate(cols):
+        check_type(column, c, 'columns[%d]' % (i,))
 
         c = copy(c)
         if c.name is None:
@@ -621,7 +644,7 @@ def data_frames(
                             seen.add(value)
                         if has_duplicate:
                             continue
-                    row = list(st.try_convert(tuple, row, 'draw(rows)'))
+                    row = list(try_convert(tuple, row, 'draw(rows)'))
 
                     if len(row) > len(rewritten_columns):
                         raise InvalidArgument((
