@@ -260,7 +260,10 @@ class ConjectureRunner(object):
             if self.valid_examples >= self.settings.max_examples:
                 self.exit_with(ExitReason.max_examples)
             if self.call_count >= max(
-                self.settings.max_iterations, self.settings.max_examples
+                self.settings.max_examples * 10,
+                # We have a high-ish default max iterations, so that tests
+                # don't become flaky when max_examples is too low.
+                1000
             ):
                 self.exit_with(ExitReason.max_iterations)
 
@@ -658,8 +661,7 @@ class ConjectureRunner(object):
                 HealthCheck.large_base_example
             )
 
-        if self.settings.perform_health_check:
-            self.health_check_state = HealthCheckState()
+        self.health_check_state = HealthCheckState()
 
         count = 0
         while not self.interesting_examples and (
@@ -1016,7 +1018,7 @@ class Negated(object):
         self.tag = tag
 
 
-NEGATED_CACHE = {}
+NEGATED_CACHE = {}  # type: dict
 
 
 def negated(tag):
@@ -1773,6 +1775,8 @@ class Shrinker(object):
             ):
                 discarded.append((ex.start, ex.end))
 
+        assert discarded
+
         attempt = bytearray(self.shrink_target.buffer)
         for u, v in reversed(discarded):
             del attempt[u:v]
