@@ -60,6 +60,11 @@ ONE_FROM_MANY_LABEL = calc_label_from_name('one more from many()')
 def integer_range(data, lower, upper, center=None):
     assert lower <= upper
     if lower == upper:
+        # Write a value even when this is trival so that when a bound depends
+        # on other values we don't suddenly disappear when the gap shrinks to
+        # zero - if that happens then often the data stream becomes misaligned
+        # and we fail to shrink in cases where we really should be able to.
+        data.write(hbytes([0]))
         return int(lower)
 
     if center is None:
@@ -103,36 +108,36 @@ def centered_integer_range(data, lower, upper, center):
     )
 
 
-def check_sample(values):
-    try:
-        from numpy import ndarray
-        if isinstance(values, ndarray):
-            if values.ndim != 1:
-                note_deprecation((
-                    'Only one-dimensional arrays are supported for sampling, '
-                    'and the given value has {ndim} dimensions (shape '
-                    '{shape}).  This array would give samples of array slices '
-                    'instead of elements!  Use np.ravel(values) to convert '
-                    'to a one-dimensional array, or tuple(values) if you '
-                    'want to sample slices.  Sampling a multi-dimensional '
-                    'array will be an error in a future version of Hypothesis.'
-                ).format(ndim=values.ndim, shape=values.shape))
-            return tuple(values)
-    except ImportError:  # pragma: no cover
-        pass
+try:
+    from numpy import ndarray
+except ImportError:  # pragma: no cover
+    ndarray = ()
 
-    if not isinstance(values, (OrderedDict, Sequence, enum.EnumMeta)):
+
+def check_sample(values, strategy_name):
+    if isinstance(values, ndarray):
+        if values.ndim != 1:
+            note_deprecation((
+                'Only one-dimensional arrays are supported for sampling, '
+                'and the given value has {ndim} dimensions (shape '
+                '{shape}).  This array would give samples of array slices '
+                'instead of elements!  Use np.ravel(values) to convert '
+                'to a one-dimensional array, or tuple(values) if you '
+                'want to sample slices.  Sampling a multi-dimensional '
+                'array will be an error in a future version of Hypothesis.'
+            ).format(ndim=values.ndim, shape=values.shape))
+    elif not isinstance(values, (OrderedDict, Sequence, enum.EnumMeta)):
         note_deprecation(
-            ('Cannot sample from %r, not a sequence.  ' % (values,)) +
-            'Hypothesis goes to some length to ensure that sampling an '
-            'element from a collection (with `sampled_from` or `choices`) is '
-            'replayable and can be minimised.  To replay a saved example, '
-            'the sampled values must have the same iteration order on every '
-            'run - ruling out sets, dicts, etc due to hash randomisation.  '
-            'Most cases can simply use `sorted(values)`, but mixed types or '
-            'special values such as math.nan require careful handling - and '
-            'note that when simplifying an example, Hypothesis treats '
-            'earlier values as simpler.')
+            'Cannot sample from {values}, not an ordered collection. '
+            'Hypothesis goes to some length to ensure that the {strategy} '
+            'strategy has stable results between runs. To replay a saved '
+            'example, the sampled values must have the same iteration order '
+            'on every run - ruling out sets, dicts, etc due to hash '
+            'randomisation. Most cases can simply use `sorted(values)`, but '
+            'mixed types or special values such as math.nan require careful '
+            'handling - and note that when simplifying an example, '
+            'Hypothesis treats earlier values as simpler.'.format(
+                values=repr(values), strategy=strategy_name))
     return tuple(values)
 
 
@@ -273,7 +278,6 @@ class Sampler(object):
        unchanged) the value.
     2. base[i] < alternate[i], so that shrinking the draw always results in
        shrinking the chosen element.
-
     """
 
     def __init__(self, weights):
@@ -361,10 +365,10 @@ class many(object):
     elements = many(data, ...)
     while elements.more():
         add_stuff_to_result()
-
     """
 
     def __init__(self, data, min_size, max_size, average_size):
+        assert 0 <= min_size <= average_size <= max_size
         self.min_size = min_size
         self.max_size = max_size
         self.data = data
