@@ -22,15 +22,29 @@ from collections import defaultdict
 import hypothesis.internal.conjecture.utils as cu
 from hypothesis.errors import NoExamples, NoSuchExample, Unsatisfiable, \
     UnsatisfiedAssumption
-from hypothesis.control import assume, reject, _current_build_context
+from hypothesis.control import assume, _current_build_context
 from hypothesis._settings import note_deprecation
 from hypothesis.internal.compat import hrange, bit_length
+from hypothesis.internal.coverage import check_function
 from hypothesis.utils.conventions import UniqueIdentifier
 from hypothesis.internal.lazyformat import lazyformat
 from hypothesis.internal.reflection import get_pretty_function_description
+from hypothesis.internal.validation import check_type
 from hypothesis.internal.conjecture.utils import LABEL_MASK, \
     combine_labels, calc_label_from_cls, calc_label_from_name
 
+try:
+    from random import Random  # noqa
+    from typing import List, Callable, TypeVar, Generic, Optional  # noqa
+    Ex = TypeVar('Ex', covariant=True)
+    T = TypeVar('T')
+
+    from hypothesis.internal.conjecture.data import ConjectureData  # noqa
+
+except ImportError:  # pragma: no cover
+    Ex = 'key'  # type: ignore
+    Generic = {Ex: object}  # type: ignore
+
 calculating = UniqueIdentifier('calculating')
 
 MAPPED_SEARCH_STRATEGY_DO_DRAW_LABEL = calc_label_from_name(
@@ -45,7 +59,7 @@ def one_of_strategies(xs):
     return OneOfStrategy(xs)
 
 
-class SearchStrategy(object):
+class SearchStrategy(Generic[Ex]):
     """A SearchStrategy is an object that knows how to explore data of a given
     type.
 
@@ -228,6 +242,7 @@ class SearchStrategy(object):
         return False
 
     def example(self, random=None):
+        # type: (Random) -> Ex
         """Provide an example of the sort of value that this strategy
         generates. This is biased to be slightly simpler than is typical for
         values from this strategy, for clarity purposes.
@@ -270,7 +285,7 @@ class SearchStrategy(object):
         # Conjecture will always try the zero example first. This would result
         # in us producing the same example each time, which is boring, so we
         # deliberately skip the first example it feeds us.
-        first = []
+        first = []  # type: list
 
         def condition(x):
             if first:
@@ -299,6 +314,7 @@ class SearchStrategy(object):
             )
 
     def map(self, pack):
+        # type: (Callable[[Ex], T]) -> SearchStrategy[T]
         """Returns a new strategy that generates values by generating a value
         from this strategy and then calling pack() on the result, giving that.
 
@@ -309,6 +325,7 @@ class SearchStrategy(object):
         )
 
     def flatmap(self, expand):
+        # type: (Callable[[Ex], SearchStrategy[T]]) -> SearchStrategy[T]
         """Returns a new strategy that generates values by generating a value
         from this strategy, say x, then generating a value from
         strategy(expand(x))
@@ -321,6 +338,7 @@ class SearchStrategy(object):
         )
 
     def filter(self, condition):
+        # type: (Callable[[Ex], bool]) -> SearchStrategy[Ex]
         """Returns a new strategy that generates values from this strategy
         which satisfy the provided condition. Note that if the condition is too
         hard to satisfy this might result in your tests failing with
@@ -335,6 +353,7 @@ class SearchStrategy(object):
 
     @property
     def branches(self):
+        # type: () -> List[SearchStrategy[Ex]]
         return [self]
 
     def __or__(self, other):
@@ -348,6 +367,7 @@ class SearchStrategy(object):
         return one_of_strategies((self, other))
 
     def validate(self):
+        # type: () -> None
         """Throw an exception if the strategy is not valid.
 
         This can happen due to lazy construction
@@ -392,6 +412,7 @@ class SearchStrategy(object):
         pass
 
     def do_draw(self, data):
+        # type: (ConjectureData) -> Ex
         raise NotImplementedError('%s.do_draw' % (type(self).__name__,))
 
     def __init__(self):
@@ -432,7 +453,6 @@ class OneOfStrategy(SearchStrategy):
 
     @property
     def element_strategies(self):
-        from hypothesis.strategies import check_strategy
         if self.__element_strategies is None:
             strategies = []
             for arg in self.original_strategies:
@@ -469,6 +489,7 @@ class OneOfStrategy(SearchStrategy):
         ])
 
     def do_draw(self, data):
+        # type: (ConjectureData) -> Ex
         n = len(self.element_strategies)
         assert n > 0
         if n == 1:
@@ -538,6 +559,7 @@ class MappedSearchStrategy(SearchStrategy):
             '%s.pack()' % (self.__class__.__name__))
 
     def do_draw(self, data):
+        # type: (ConjectureData) -> Ex
         for _ in range(3):
             i = data.index
             try:
@@ -549,10 +571,11 @@ class MappedSearchStrategy(SearchStrategy):
                 data.stop_example(discard=True)
                 if data.index == i:
                     raise
-        reject()
+        raise UnsatisfiedAssumption()
 
     @property
     def branches(self):
+        # type: () -> List[SearchStrategy[Ex]]
         return [
             MappedSearchStrategy(pack=self.pack, strategy=strategy)
             for strategy in self.mapped_strategy.branches
@@ -584,6 +607,7 @@ class FilteredStrategy(SearchStrategy):
         self.filtered_strategy.validate()
 
     def do_draw(self, data):
+        # type: (ConjectureData) -> Ex
         for i in hrange(3):
             start_index = data.index
             value = data.draw(self.filtered_strategy)
@@ -601,10 +625,17 @@ class FilteredStrategy(SearchStrategy):
             self,
         ))
         data.mark_invalid()
+        raise AssertionError('Unreachable, for Mypy')  # pragma: no cover
 
     @property
     def branches(self):
+        # type: () -> List[SearchStrategy[Ex]]
         return [
             FilteredStrategy(strategy=strategy, condition=self.condition)
             for strategy in self.filtered_strategy.branches
         ]
+
+
+@check_function
+def check_strategy(arg, name=''):
+    check_type(SearchStrategy, arg, name)
